@@ -24,7 +24,7 @@ keywords = [
   # Helpers
   # -------
   # variable
-  "with",
+  "with", "tagged", "when",
   # other
   "has", "of", "to", "as", "where",
   # figures
@@ -155,7 +155,7 @@ class Lexer(object):
 
     def t_REGEX(self, t):
         r'\/[^\s]+\/i?'
-        re.compile(t.value[1:-2])
+        t.value = dict(regexp=re.compile(t.value[1:-1]).pattern)
         return t
 
     def t_NUMBEROF(self, t):
@@ -193,6 +193,7 @@ class Lexer(object):
 
     def t_ISNT(self, t):
         r"((is\snot)|isnt|not)"
+        t.value = 'isnot'
         return t
 
     def t_CONJ(self, t):
