@@ -12,7 +12,6 @@ except ImportError: # pragma: no cover
     lextab, yacctab = 'lextab', 'yacctab'
 
 
-
 class Parser(object):
     def __init__(self, lex_optimize=False, lextab=lextab, yacc_optimize=False, yacctab=yacctab, yacc_debug=False):
         self.lexer = Lexer()
@@ -21,28 +20,20 @@ class Parser(object):
         self.parser = yacc.yacc(module=self, tabmodule=yacctab, start="program",
                                 outputdir=os.path.dirname(__file__), optimize=yacc_optimize, debug=yacc_debug)
 
-    def parse(self, source, filename="", debug=0):
-        self.debug = debug!=0
+    def parse(self, source, debug=False, using_cli=False):
+        self.debug = debug
         self.errors = []
-        
-        if self.errors:
-            raise EnvironmentError("Parser was already used")
+        self.using_cli = using_cli
         task = self.parser.parse(source, lexer=self.lexer, debug=debug)
-        if task is None or self.errors or task.errors:
-            if task and task.errors:
-                for l, error in task.errors.items():
-                    self.errors.append(dict(lineno=l, message=str(error)))
+        if task is None or self.errors:
             raise ScriptError(self.errors)
         else:
             return task
 
     def p_error(self, t):
-        if t:
-            self.errors.append(dict(lineno=t.lineno, token=t.type, value=t.value))
-            # print "\033[91mScript Error\033[0m Line: \033[94m%s\033[0m, Token: \033[94m%s\033[0m, value: \033[94m%s\033[0m\n", 
-            # "\x1b[31mE: line: %d, Syntax Error, token: `%s`, `%s`\x1b[0m\n"
-            sys.stdout.write("\033[91mSyntax Error:\033[0m \033[96mLine:\033[0m %s, \033[96mToken:\033[0m %s, \033[96mValue:\033[0m %s\n"
-                             % (t.lineno, t.type, t.value))
+        self.errors.append(dict(lineno=t.lineno, token=t.type, value=t.value))
+        if self.using_cli:
+            sys.stdout.write("\033[91mSyntax Error:\033[0m \033[96mLine:\033[0m %s, \033[96mToken:\033[0m %s, \033[96mValue:\033[0m %s\n"% (t.lineno, t.type, t.value))
 
         self.parser.restart()
         return t
@@ -158,13 +149,16 @@ class Parser(object):
                 | WEEKLY     juice
                 | MONTHLY    juice
                 | QUARTERLY  juice
-                | YEARLY     juice
-                | NI         juice'''
+                | YEARLY     juice'''
         p[0] = ast.Method('schedule', self, p.lineno(1), p[2][0], args=[p[1]] + p[2][1], kwargs=p[2][2])
 
+    def p_task_when(self, p):
+        '''stmt : WHEN figure_expression  suite'''
+        p[0] = ast.Method("when", parser=self, lineno=p.lineno(1), suite=p[3], args=(p[2], ))
+
     def p_task_after(self, p):
-        '''stmt : AFTER paths suite'''
-        p[0] = ast.Method("after", parser=self, lineno=p.lineno(1), suite=p[3], args=(p[2].path.replace(".","-"), ))
+        '''stmt : AFTER string output suite'''
+        p[0] = ast.Method("after", parser=self, lineno=p.lineno(1), suite=p[4], args=(p[2], ))
 
     def p_methods(self, p):
         '''stmt : PATH juice'''
@@ -234,7 +228,7 @@ class Parser(object):
         if len(p) == 2:
             p[0] = dict(unique=True, lineno=p.lineno(1))
 
-    def p_figure_expression(self, p):
+    def p_figexp(self, p):
         '''figexp : agg expression
                   |     expression'''
         if len(p) == 3:
@@ -244,7 +238,7 @@ class Parser(object):
             p[0] = p[1]
 
     def p_figure_expressions(self, p):
-        '''figexps : WHERE figexp
+        '''figexps : WHERE       figexp
                    | figexps AND figexp
                    | figexps OR  figexp'''
         if len(p) == 4:
@@ -330,6 +324,14 @@ class Parser(object):
         _.pop('lineno', None)
         p[0] = ast.Figure(self, _.pop('paths'), **_)
 
+    def p_figure_expression(self, p):
+        '''figure_expression : figure LT expression
+                             | figure GT expression
+                             | figure EQ expression
+                             | figure NE expression'''
+        p[0] = ast.Expression(p[1])
+        p[0].add(p[2], p[3])
+
     # ---------------
     # Methods w/ data
     # ---------------
@@ -371,10 +373,8 @@ class Parser(object):
     # While Loops
     # -----------
     def p_stmt_while(self, p):
-        '''stmt : WHILE paths output suite
-                | WHILE expressions output suite'''
-        #         soon...
-        #       | WHILE figure suite
+        '''stmt : WHILE paths               output suite
+                | WHILE expressions         output suite'''
         p[0] = ast.Method("while", parser=self, lineno=p.lineno(1), output=p[3], suite=p[4], args=(p[2], ))
 
     # -----------
@@ -408,6 +408,10 @@ class Parser(object):
         '''expression : paths HAS paths'''
         p[0] = ast.Expression(ast.Comparison(p[1], "has", p[3]))
 
+    def p_expression_is_isnt(self, p):
+        '''expression : ISNT expression'''
+        p[0] = ast.Expression(ast.Comparison(p[2], p[1], True))
+
     def p_expression_not_in(self, p):
         '''expression : paths ISNT NI paths'''
         p[0] = ast.Expression(ast.Comparison(p[4], "excludes", p[1]))
@@ -420,18 +424,33 @@ class Parser(object):
         else:
             p[0] = ast.Expression(ast.Comparison(p[1], "contains", p[3]))
 
+    def p_expression_like(self, p):
+        '''expression : paths      LIKE expression
+                      | paths IS   LIKE expression
+                      | paths ISNT LIKE expression
+                      | paths      LIKE REGEX
+                      | paths IS   LIKE REGEX
+                      | paths ISNT LIKE REGEX'''
+        if len(p) == 5:
+            if p[2] == 'isnot':
+                p[0] = ast.Expression(ast.Comparison(p[1], "notlike", p[4]))
+            else:
+                p[0] = ast.Expression(ast.Comparison(p[1], "like", p[4]))
+        else:
+            p[0] = ast.Expression(ast.Comparison(p[1], "like", p[3]))
+
     def p_expression_is(self, p):
         '''expression : paths IS   variable
                       | paths ISNT variable'''
         p[0] = ast.Expression(ast.Comparison(p[1], "is" if p[2]=='is' else 'isnt', p[3]))
 
-    def p_expression_likere(self, p):
-        '''expression : expression LIKE REGEX'''
-        p[0] = ast.Expression(ast.Comparison(p[1], "like", dict(regexp=p[3])))
-
-    def p_expression_likestr(self, p):
-        '''expression : expression LIKE expression'''
-        p[0] = ast.Expression(ast.Comparison(p[1], "like", p[3]))
+    def p_expression_tagged(self, p):
+        '''expression : ISNT TAGGED string
+                      |      TAGGED string'''
+        if len(p)==4:
+            p[0] = ast.Expression(ast.Tagged(p[3], False))
+        else:
+            p[0] = ast.Expression(ast.Tagged(p[2], True))
 
     # ------------------
     # Expressions > Math
@@ -499,7 +518,7 @@ class Parser(object):
                  | KWARG variable NEWLINE
                  | KWARG paths NEWLINE'''
         if len(p) == 3:
-            p[0] = {p[1][2:]: None}
+            p[0] = {p[1][2:]: True}
         else:
             p[0] = {p[1][2:]: p[2]}
 
